@@ -513,7 +513,7 @@ g_date_fill_parse_tokens (const gchar *str, GDateParseTokens *pt)
     {
       
       i = 0;
-      while (*s && g_ascii_isdigit (*s) && i <= NUM_LEN)
+      while (*s && g_ascii_isdigit (*s) && i < NUM_LEN)
         {
           num[pt->num_ints][i] = *s;
           ++s; 
@@ -924,7 +924,7 @@ g_date_set_time_t (GDate *date,
  *
  * Sets the value of a date from a #GTime value. 
  *
- * @Deprecated: Use g_date_set_time_t() instead.
+ * @Deprecated:2.10: Use g_date_set_time_t() instead.
  */
 void
 g_date_set_time (GDate    *date,
