@@ -27,11 +27,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include "galias.h"
 #include "glib.h"
 #include "gprintf.h"
 #include "gprintfint.h"
 
+#include "galias.h"
+
 /**
  * g_printf:
  * @format: a standard printf() format string, but notice 
@@ -337,6 +338,5 @@ g_vasprintf (gchar      **string,
   return len;
 }
 
-
-
-
+#define __G_PRINTF_C__
+#include "galiasdef.c"
