@@ -576,6 +576,11 @@ g_parse_debug_string  (const gchar     *string,
   guint result = 0;
   
   g_return_val_if_fail (string != NULL, 0);
+
+  /* this function is used by gmem.c/gslice.c initialization code,
+   * so introducing malloc dependencies here would require adaptions
+   * of those code portions.
+   */
   
   if (!g_ascii_strcasecmp (string, "all"))
     {
@@ -586,23 +591,21 @@ g_parse_debug_string  (const gchar     *string,
     {
       const gchar *p = string;
       const gchar *q;
-      gboolean done = FALSE;
       
-      while (*p && !done)
+      while (*p)
 	{
 	  q = strchr (p, ':');
 	  if (!q)
-	    {
-	      q = p + strlen(p);
-	      done = TRUE;
-	    }
+	    q = p + strlen(p);
 	  
-	  for (i=0; i<nkeys; i++)
+	  for (i = 0; i < nkeys; i++)
 	    if (g_ascii_strncasecmp (keys[i].key, p, q - p) == 0 &&
 		keys[i].key[q - p] == '\0')
 	      result |= keys[i].value;
 	  
-	  p = q + 1;
+	  p = q;
+	  if (*p == ':')
+	    p++;
 	}
     }
   
@@ -1155,6 +1158,30 @@ g_getenv (const gchar *variable)
 #endif /* G_OS_WIN32 */
 }
 
+/* _g_getenv_nomalloc
+ * this function does a getenv() without doing any kind of allocation
+ * through glib. it's suitable for chars <= 127 only (both, for the
+ * variable name and the contents) and for contents < 1024 chars in
+ * length. also, it aliases "" to a NULL return value.
+ **/
+const gchar*
+_g_getenv_nomalloc (const gchar *variable,
+                    gchar        buffer[1024])
+{
+  const gchar *retval = getenv (variable);
+  if (retval && retval[0])
+    {
+      gint l = strlen (retval);
+      if (l < 1024)
+        {
+          strncpy (buffer, retval, l);
+          buffer[l] = 0;
+          return buffer;
+        }
+    }
+  return NULL;
+}
+
 /**
  * g_setenv:
  * @variable: the environment variable to set, must not contain '='.
@@ -1377,6 +1404,7 @@ g_unsetenv (const gchar *variable)
 gchar **
 g_listenv (void)
 {
+#ifndef G_OS_WIN32
   gchar **result, *eq;
   gint len, i, j;
 
@@ -1394,6 +1422,73 @@ g_listenv (void)
   result[j] = NULL;
 
   return result;
+#else
+  gchar **result, *eq;
+  gint len = 0, i, j;
+
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *p, *q;
+
+      p = (wchar_t *) GetEnvironmentStringsW ();
+      if (p != NULL)
+	{
+	  q = p;
+	  while (*q)
+	    {
+	      q += wcslen (q) + 1;
+	      len++;
+	    }
+	}
+      result = g_new0 (gchar *, len + 1);
+
+      j = 0;
+      q = p;
+      while (*q)
+	{
+	  result[j] = g_utf16_to_utf8 (q, -1, NULL, NULL, NULL);
+	  if (result[j] != NULL)
+	    {
+	      eq = strchr (result[j], '=');
+	      if (eq && eq > result[j])
+		{
+		  *eq = '\0';
+		  j++;
+		}
+	      else
+		g_free (result[j]);
+	    }
+	  q += wcslen (q) + 1;
+	}
+      result[j] = NULL;
+      FreeEnvironmentStringsW (p);
+    }
+  else
+    {
+      len = g_strv_length (environ);
+      result = g_new0 (gchar *, len + 1);
+      
+      j = 0;
+      for (i = 0; i < len; i++)
+	{
+	  result[j] = g_locale_to_utf8 (environ[i], -1, NULL, NULL, NULL);
+	  if (result[j] != NULL)
+	    {
+	      eq = strchr (result[j], '=');
+	      if (eq && eq > result[j])
+		{
+		  *eq = '\0';
+		  j++;
+		}
+	      else
+		g_free (result[j]);
+	    }
+	}
+      result[j] = NULL;
+    }
+
+  return result;
+#endif
 }
 
 G_LOCK_DEFINE_STATIC (g_utils_global);
