@@ -19,9 +19,9 @@
 
 #include <stdlib.h>
 
-#include "gobjectalias.h"
 #include "gtypeplugin.h"
 #include "gtypemodule.h"
+#include "gobjectalias.h"
 
 typedef struct _ModuleTypeInfo ModuleTypeInfo;
 typedef struct _ModuleInterfaceInfo ModuleInterfaceInfo;
@@ -123,7 +123,7 @@ g_type_module_get_type (void)
         NULL,               /* interface_data */
       };
 
-      type_module_type = g_type_register_static (G_TYPE_OBJECT, "GTypeModule", &type_module_info, G_TYPE_FLAG_ABSTRACT);
+      type_module_type = g_type_register_static (G_TYPE_OBJECT, g_intern_static_string ("GTypeModule"), &type_module_info, G_TYPE_FLAG_ABSTRACT);
 
       g_type_add_interface_static (type_module_type, G_TYPE_TYPE_PLUGIN, &iface_info);
     }
@@ -424,3 +424,7 @@ g_type_module_register_flags (GTypeModule      *module,
   return g_type_module_register_type (G_TYPE_MODULE (module),
                                       G_TYPE_FLAGS, name, &flags_type_info, 0);
 }
+
+
+#define __G_TYPE_MODULE_C__
+#include "gobjectaliasdef.c"
