@@ -13,6 +13,9 @@ echo_v ()
   fi
 }
 
+LC_ALL=en_US
+export LC_ALL
+
 error_out=/dev/null
 if [ "$1" = "-v" ]; then
   verbose=1
